@@ -2,7 +2,7 @@
 
 __author__ = """PragmaLingu"""
 __email__ = 'info@pragmalingu.de'
-__version__ = '0.0.3'
+__version__ = '0.0.9'
 
 from collections import OrderedDict, defaultdict
 import pandas as pd
@@ -46,8 +46,8 @@ class EvaluationObject:
 
     def check_searched_queries(self, query_ids):
         """
-        Checks if query ids an int or None and transforms it to a list.
-        If it's None, all queries are searched.
+        Checks if query_ids is an int or None and transforms it to a list.
+        If it's None, all available queries are used for the search.
         :param query_ids: can be a list, an int or None
         :return: transformed query ids
         """
@@ -116,7 +116,7 @@ class EvaluationObject:
                 "true_positives": []
             }
             result = self.get_search_result(query_ID, size, fields)
-            for pos, hit in enumerate(result["hits"]["hits"]):
+            for pos, hit in enumerate(result["hits"]["hits"], start=1):
                 # check if `hit` IS a relevant document; in case `hits` position < k, it counts as a true positive;
                 if int(hit["_id"]) in self.queries_rels[query_ID]['relevance_assessments'] and pos <= k:
                     true = self.create_hit(pos, hit, fields)
@@ -147,7 +147,7 @@ class EvaluationObject:
             }
             result = self.get_search_result(query_ID, size, fields)
             # for every `hit` in the search results... ;
-            for pos, hit in enumerate(result["hits"]["hits"]):
+            for pos, hit in enumerate(result["hits"]["hits"], start=1):
                 # check if `hit` IS a relevant document; in case `hits` position < k, it counts as a true positive;
                 if int(hit["_id"]) not in self.queries_rels[query_ID]['relevance_assessments'] and pos < k:
                     false = self.create_hit(pos, hit, fields)
@@ -179,7 +179,7 @@ class EvaluationObject:
             result = self.get_search_result(query_ID, size, fields)
             # iterating through the results;
             query_rel = self.queries_rels[query_ID]['relevance_assessments'].copy()
-            for pos, hit in enumerate(result["hits"]["hits"]):
+            for pos, hit in enumerate(result["hits"]["hits"], start=1):
                 # false negatives require that the result belongs to the relevance assessments;
                 if int(hit["_id"]) in query_rel:
                     if pos > k:
@@ -294,6 +294,10 @@ class EvaluationObject:
         :param fn: int, false negatives
         :return: Recall value
         """
+        if (tp + fn) == 0:
+            raise ValueError('Sum of true positives and false negatives is 0. Please check your data, '
+                             'this shouldn\'t happen. Maybe you tried searching on the wrong index, with the wrong '
+                             'queries or on the wrong fields.')
         return tp / (tp + fn)
 
     def calculate_precision(self, tp, fp):
@@ -303,6 +307,10 @@ class EvaluationObject:
         :param fn: int, false negatives
         :return: Precision value
         """
+        if (tp + fp) == 0:
+            raise ValueError('Sum of true positives and false positives is 0. Please check your data, '
+                             'this shouldn\'t happen. Maybe you tried searching on the wrong index, with the wrong '
+                             'queries or on the wrong fields.')
         return tp / (tp + fp)
 
     def calculate_fscore(self, precision, recall, factor=1):
@@ -420,9 +428,12 @@ class EvaluationObject:
                     }
                 }
             }
-        explanation = self.elasticsearch.explain(self.index, doc_id, query_body)['explanation']
         explain = defaultdict(lambda: defaultdict(lambda: []))
+        explanation = self.elasticsearch.explain(self.index, doc_id, query_body)['explanation']
         explain["score"] = explanation['value']
+        if len(fields) < 2:
+            explanation = {'details': [explanation]}
+
         for el in explanation['details']:
             field = ''.join(f for f in fields if re.search(f, el['details'][0]['description']))
             explain[field]["total_value"] = el['details'][0]['value']
@@ -437,7 +448,7 @@ class EvaluationObject:
                     except IndexError:
                         continue
                     try:
-                        if re.match(r'[Ff]req', val["details"][0]["description"]):
+                        if re.match(r'.*[Ff]req', val["details"][0]["description"]):
                             term_freq = val["details"][0]["value"]
                     except IndexError:
                         continue
@@ -451,9 +462,9 @@ class EvaluationObject:
 
 
 class ComparisonTool:
-    def __init__(self, host, index_1, index_2, eval_obj_1=None, eval_obj_2=None, name_1='approach_1',
-                 name_2='approach_2', qry_rel_dict=dict(),
-                 fields=['text', 'title'], size=20, k=20):
+    def __init__(self, host, qry_rel_dict, eval_obj_1=None, eval_obj_2=None,
+                 fields=['text', 'title'], index_1=None, index_2=None, name_1='approach_1',
+                 name_2='approach_2', size=20, k=20):
         if eval_obj_1 is None:
             eval_obj_1 = EvaluationObject(host, qry_rel_dict, index_1, name_1)
         if eval_obj_2 is None:
@@ -490,10 +501,11 @@ class ComparisonTool:
         else:
             setattr(self, diff_name, diff_ordered)
 
-    def get_disjoint_sets(self, distribution):
+    def get_disjoint_sets(self, distribution, highest=False):
         """
         Returns the disjoint sets of the given distribution
         :param distribution: str, possible arguments are 'false_positives' and 'false_negatives'
+        :param highest: if True it returns the set with the highest count of disjoints
         :return: a dict with disjoint lists for each approach in a dictionary for each query regarding the distribution
         """
         results = defaultdict(dict)
@@ -515,8 +527,13 @@ class ComparisonTool:
                            getattr(self.eval_obj_1, distribution)[query][distribution]):
                     results[query][distribution + ' ' + self.eval_obj_2.name].append(res_2)
             results[query]['count'] = len(results[query][distribution + ' ' + self.eval_obj_1.name])+len(results[query][distribution + ' ' + self.eval_obj_2.name])
-        ordered_results = OrderedDict(sorted(results.items(), key=lambda i: i[1]['count']))
-        return ordered_results
+        filtered_results = {key: val for key, val in results.items() if val['count'] != 0}
+        ordered_results = OrderedDict(sorted(filtered_results.items(), key=lambda i: i[1]['count']))
+        if not highest:
+            return ordered_results
+        else:
+            elements = list(ordered_results.items())
+            return elements[-1]
 
     def print_vis(self, params=None):
         """
